C--------------------------------------------------------------------
C      calculate symmety equivalent reflections and thus the multiplicity
C      of the reflections given
C--------------------------------------------------------------------
C
      SUBROUTINE calcmul(iwhat,mref,nref,nref1,F,IDX,MULT,
     $ IROT,nsym,ncen,RTRS,pi) 
C
      implicit none
      integer iwhat,mref,nref,nref1,newref
      real*8 F(2,mref)
      integer IDX(3,mref),MULT(mref),iref
      integer j,k,mm,jh,jk,jl,hh(48),kk(48),ll(48),inew,jjh,jjk,jjl
      integer IROT(3,3,48),nsym,ncen,is,ic,i
      real*8 RTRS(3,48)
      real*8 cphi,sphi,phi,pi,twopi
C
C     *** All reflections with A,B=0 are removed from the list
C     *** only if (iwhat=0):
C     *** to each given reflection F_H all symmetry equivalents are calculated
C     *** using the known symmetry operators (IROT,RTRS). The new (h'k'l')
C     *** is got by use of the rotation matrix R alone: H' = HR.
C     *** The absolute values |F_HR| are constant since no anomalous dispersion 
C     *** is considered. The phases of F_HR change if the symmetry operator has
C     *** a translatory component T not zero:   
C     ***             | r11  r12  r13 |
C     ***   ( h k l ) | r21  r22  r23 |  = (h' k' l') = H R = (rotated hkl)
C     ***             | r31  r32  r33 |
C     ***               
C     ***   | F_HR |  =  | F_H |
C     ***
C     ***   phi_HR = phi_H - 2 pi H*T ,  H=(h,k,l), T=(tx,ty,tz).
C     ***
C     *** see Giacovazzo, Fundamentals of Crystallography (1992) p. 156.
C     ***
C     *** IROT(i,j,isym) contains the rotation-matrix, RTRS(i,isym)  the 
C     *** corresponding translation
C
      twopi=2*pi
C
C     print *,'DISPLAY OF SYMMETRY INFORMATION'
C     write(*,'(a,i4)') ' Number of symm. operators given:',nsym
C     if (ncen.eq.0) print *,'No center of inversion present'
C     if (ncen.eq.1) print *,'Center of inversion present'
C     print *,'IROT(3,3,nsym), RTRS(3,nsym):'
C     write(*,'(3(3i4,2x),2x,3f5.3)') 
C    $  (((IROT(k,j,is),j=1,3),k=1,3),
C    $  (RTRS(k,is),k=1,3), is=1,nsym)
C     print *,'Enter <RETURN> to continue'
C     read(*,'(BN,i1)') j
C
C     *** 1. leave away all reflections with zero intensity ***
C     ***    save new list in old array                     ***
      newref=0
      do 80 j=1,nref
	if (F(1,j).ne.0.000 .or. F(2,j).ne.0.00) then
	   newref=newref+1
	   if (newref.ne.j) then
	     IDX(1,newref)=IDX(1,j)
	     IDX(2,newref)=IDX(2,j)
	     IDX(3,newref)=IDX(3,j)
             F(1,newref)=F(1,J)
             F(2,newref)=F(2,J)
	   endif
	endif
80    continue
      nref=newref
      write(*,*) nref,' reflections with I > 0'
      if (iwhat.ne.0) return
C
C     *** 2.  get equivalents to a given reflection
C
C      open (20,file='l1',STATUS='UNKNOWN')
C
C     -------------------------------------------------
C
      do 100 iref=1,nref
C
	jh=IDX(1,iref)
	jk=IDX(2,iref)
	jl=IDX(3,iref)
	mm=0

        DO 90 IC=1,NCEN+1
        DO 90 IS=1,NSYM  
          jjh = IROT(1,1,is)*jh+IROT(2,1,is)*jk+IROT(3,1,is)*jl 
          jjk = IROT(1,2,is)*jh+IROT(2,2,is)*jk+IROT(3,2,is)*jl 
          jjl = IROT(1,3,is)*jh+IROT(2,3,is)*jk+IROT(3,3,is)*jl 
	  if (ic.eq.2) then
             jjh=-jjh
             jjk=-jjk
             jjl=-jjl
          endif
	  inew=1
C         *** does this (hkl) already exist? ***
C         *** a) look in equivalents list of this reflection:
	  do 95 k=1,mm
	    if (jjh.eq.HH(k) .and.jjk.eq.KK(k).and.
     $          jjl.eq.LL(k)) then
              inew=0
	      goto 96
	    endif
 95       continue
C         *** b) look in global reflection list:
	  do 94 k=1,newref-mm
	    if (jjh.eq.IDX(1,k) .and.jjk.eq.IDX(2,k).and.
     $          jjl.eq.IDX(3,k).and.k.ne.iref) then
              inew=0
              print *,'error in calcul. of equiv. reflections!!'
              print *,jjh,jjk,jjl,'=',(IDX(i,k),i=1,3)
              print *,'Input RETURN'
              read (*,'(BN,i2)') i
	      goto 96
	    endif
 94       continue

 96       if (inew.eq.1) then
C           *** ok, new (hkl):  memorize and calculate F ***
C           *** translations RTRS are in relative coordinates 0...1 ***
	    mm=mm+1
	    HH(mm)=jjh
	    KK(mm)=jjk
	    LL(mm)=jjl
	    if (mm.gt.1) then
C           *** only if not original reflection H already in list ***
	    newref=newref+1
	    if (newref.gt.mref) call proend('Too  many reflections.')
	    IDX(1,newref) = jjh
	    IDX(2,newref) = jjk
	    IDX(3,newref) = jjl
	    phi=-twopi*(jh*RTRS(1,is)+jk*RTRS(2,is)+jl*RTRS(3,is))
	    cphi=dcos(phi)
	    sphi=dsin(phi)
            F(1,newref) = F(1,iref)*cphi - F(2,iref)*sphi
            F(2,newref) = F(1,iref)*sphi + F(2,iref)*cphi
            endif
          endif
 90     continue
	MULT(j)=mm 
C
C 10001   format(3i3,1x,i3,2(1x,f15.2))
C        write(20,10001) jh,jk,jl,mm,F(1,iref),F(2,iref)
C
100   continue
C     -------------------------------------------------
C     write(*,*) nref,
C    $ ' reflections before calculating symmetry equivalents'
      nref=newref
      nref1=newref
      write(*,*) nref,
     $ ' reflections after calculating symmetry equivalents'
C      close(20)
C
C      open (20,file='l2',STATUS='UNKNOWN')
C      do 110 k=1,nref
C        write(20,*) (IDX(j,k),j=1,3),(F(j,k),j=1,2)
C110   continue
C      close(20)
      RETURN
      END
