
C--------------------------------------------------------------------
C     calculate an arbitrary plane given by three points
C--------------------------------------------------------------------
C
      SUBROUTINE fcalc2(mref,nref,IDX,F,rdint,av,pi,title)
C
C     *** rho(x,y,z)=1/Vc * sum_h sum_k sum_l (A cos(phi) + B sin(phi)
C     *** phi = 2 pi (hx + ky + lz)  see Int. Tables I (1969), p. 354 
C
C     *** mref = max. number of reflections
C     *** nref = number of reflections
C     *** F()  = array with real and imaginary part of structure factors
C     *** rdint/av = total charge (in electrons) in unit cell.  
C
      IMPLICIT  none
      integer mref,nref
      real*8 rdint,av,pi,twopi,x,y,z,rd,phas
      real*8 F(2,mref)
      real*8 eck1x,eck1y,eck1z,eck2x,eck2y,eck2z,eck3x,eck3y,eck3z
      real*8 d1x,d1y,d1z,d2x,d2y,d2z
      integer IDX(3,mref),nx,ny
      integer nadd,nyes,j,i,k
      character title*200,filden*50
 2010 FORMAT(BN,A30)
 2020 FORMAT(BN,I5)

      open (20,file='l4',STATUS='UNKNOWN')
      do 110 i=1,nref
        write(20,*) (IDX(j,i),j=1,3),(F(j,i),j=1,2)
110   continue
      close(20)

      write(*,*) 'Please input three corners in relative ',
     $  'cell coordinates'
      print *,'Order of points is:'
      print *,'1. lower left'
      print *,'2. lower right and'
      print *,'3. upper left corner of your map.'
      print *
      write(*,*) '1. Lower left corner: x,y,z'
      read(*,*) eck1x, eck1y,eck1z
      write(*,*) '2. Lower right corner: x,y,z'
      read(*,*) eck2x, eck2y,eck2z
      write(*,*) '3. Upper left corner: x,y,z'
      read(*,*) eck3x, eck3y,eck3z
      write(*,*) 'How many pixels along x-axis (2-1) ?'
      read(*,2020) nx
      write(*,*) 'How many pixels along y-axis (3-1) ?'
      read(*,2020) ny
      nadd=0
303   write(*,*) ' Should F(000) be added ? (0=yes)'
      read(*,'(BN,I5)',ERR=303) nadd
      if (nadd.eq.0) then 
	 print *, 'F(000)=', rdint*AV, 
     $            ' Ok ? (0=yes)'
         read(*,'(BN,I5)',ERR=303) nyes
         if (nyes.ne.0) then
	    print *,'Please input F(000):'
	    read(*,*,ERR=303) rdint
	    rdint=rdint/AV
            print *, 'To the density ', rdint, ' is added.'
	 endif
      endif

1303  format (a,3(f9.5,1x))
304   write(*,*) 'File-name for your density map ?'
      read(*,2010) filden
      open(20,FILE=filden,STATUS='UNKNOWN',ERR=304)
      write(20,*) 'ix  iy   density'
      write(20,*) 'FOURIER: ',title(1:180)
      write(20,1303) 'Corner1: ',eck1x,eck1y,eck1z
      write(20,1303) 'Corner2: ',eck2x,eck2y,eck2z
      write(20,1303) 'Corner3: ',eck3x,eck3y,eck3z
      print *
      print *,'Calculation in progress, please wait ...'
      twopi=pi*2
      d1x=(eck2x-eck1x)/dfloat(nx)
      d1y=(eck2y-eck1y)/dfloat(nx)
      d1z=(eck2z-eck1z)/dfloat(nx)
      d2x=(eck3x-eck1x)/dfloat(ny)
      d2y=(eck3y-eck1y)/dfloat(ny)
      d2z=(eck3z-eck1z)/dfloat(ny)
      do 210 j=0,ny
      do 210 i=0,nx
	 rd=0.0d0
         x=eck1x + i*d1x + j*d2x	
         y=eck1y + i*d1y + j*d2y	
         z=eck1z + i*d1z + j*d2z	
	 do 208 k=1,nref
	   phas=twopi*(idx(1,k)*x + idx(2,k)*y + idx(3,k)*z)
           rd=rd + F(1,k)*dcos(phas) + F(2,k)*dsin(phas)
208     continue
        rd=rd/av
        if (nadd.eq.0) rd = rd + rdint
C       write(20,'(i4,1x,i4,2x,f15.5)') i,j,rd
        write(20,*) i,j,sngl(rd)
210   continue
      close(20)
      return 
      end
