C--------------------------------------------------------------------
C     SOLVE EQUATION (10) IN REFERENCE.
C calculate the derivatives dC/drho for F- and G-constraints
C--------------------------------------------------------------------
C 
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints (F=A+iB)
C*   SIGF(1..nrefF) contains the sigmas(|F|) of the F-constraints
C*   GOBS(1..nrefG) contains the mean |F| of G-constraints
C*     SIGG(1..nrefG) contains the sigma of Gobs of group j
C*     NGRP(1..nrefG) contains the number of reflections in group j
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j
C*   nrefF,nrefG contain the numbers of constraints: F, G
C*   nref=total number of reflections>=nrefF+nrefG
C*   FCAL(1..2,1..nref) contains the calculated A and B of ALL constraints
C*   GCAL(1..nrefG) contains the calculated |G| of the G-constraints

C
      SUBROUTINE MAXENT(ngsw,mref,nrefF,nrefG,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,RMIC,tnref,lambda
CENT $  ,RMIS
     $  ,PMUL,PMULS,RDEO,RDEN,RHELP1,RHELP2,RHELP3)
C
      IMPLICIT none
      integer ngsw,mref,jref,iref,nrefF,nrefG,igrp
      integer*4 masym,nasym,ia
C     *** density arrays ***
      real*8 RDEN(masym),RDEO(masym),rtasym
C     *** structure factor lists ***
      real*8 FOBS(2,mref),FCAL(2,mref),SIGF(mref),PROF(mref),
     $         GOBS(mref),GCAL(mref),SIGG(mref)
      integer NGRP(mref)
      real*8 PMUL(mref),PMULS(mref)
C     *** total number of Fobs/Gobs given 
      real*8 tnref    
C     *** cosine and sinus-tables ***
      real*4 RMIC(mref,masym)
CENT  real*4 RMIS(mref,masym)
C     *** auxilary arrays ***
      real*8 RHELP1(mref),RHELP2(mref),RHELP3(mref) 
      real*8 sum1,lambda,derF,derG
C
C     *** precalculations ***
C
      do 1 iref=1,nrefF
        RHELP1(iref)= (FCAL(1,iref)*PROF(iref)-FOBS(1,iref))
     $	          /(SIGF(iref)*rtasym)
CENT    RHELP2(iref)= (FCAL(2,iref)*PROF(iref)-FOBS(2,iref))
CENT $            /(SIGF(iref)*rtasym)
1     continue
C     *** this part only, if groups of reflections given (G-constraint) ***
      IF (ngsw.EQ.0) THEN
        jref=nrefF+1
        DO 3 iref=1,nrefG
          DO 2 igrp=1,NGRP(iref)
            RHELP1(jref)=PMUL(jref-nrefF)*FCAL(1,jref)*PROF(jref)
CENT        RHELP2(jref)=PMUL(jref-nrefF)*FCAL(2,jref)*PROF(jref)
            jref=jref+1
2         continue
          RHELP3(iref)= (GCAL(iref)-GOBS(iref))/
     $          (GCAL(iref)*SIGG(iref)*PMULS(iref)*rtasym)
3       continue
      ENDIF
C     *** end precalculations ***
C
C     *** now calculate derivatives for each point in asymmetric unit ***
C
      derF=0.0
      derG=0.0
      DO 10 ia=1,nasym
C        *** DERIVATIVE FOR F-TYPE CONSTRAINT ***
         derF=0.0d0
         DO 20 iref=1,nrefF
           derF=derF + RHELP1(iref)*RMIC(iref,ia)
CENT $               + RHELP2(iref)*RMIS(iref,ia)
   20    CONTINUE
         if (nrefF.gt.0) derF=derF/nrefF
C        *** DERIVATIVE FOR G-TYPE CONSTRAINT ***
         IF (ngsw.EQ.0) THEN
         derG=0.0d0
         jref=nrefF+1
         DO 40 iref=1,nrefG
           sum1=0.0
           DO 50 igrp=1,NGRP(iref)
             sum1=sum1 + RHELP1(jref)*RMIC(jref,ia)
CENT $                 + RHELP2(jref)*RMIS(jref,ia)
             jref=jref+1
   50      CONTINUE
           derG=derG+sum1*RHELP3(iref)
   40    CONTINUE
         if (nrefG.gt.0) derG=derG/nrefG
         END IF
C
C*** SOLVE EQ.(10) IN REFERENCE. ***
C
         RDEN(ia)=RDEO(ia)*DEXP(-lambda*(derF+derG))
C
   10 CONTINUE
C
      RETURN
      END
