C--------------------------------------------------------------------
C     MAKE ASYMMETRIC UNIT CELL COORDINATE. ( PIXEL UNIT )
C     AND PHASE FACTOR TABLES
C--------------------------------------------------------------------
C
      SUBROUTINE MKASYM(isw,masym,nasym,mpix,NPIX,
     $  IROT,RTRS,RSUB,nsym,ncen,nsub,
     $  RDEN,rtasym,RASYM,rdint,IASYM,RMIC,
CENT $  RMIS,
     $  IDX,mref,nref,IDE,pi)
C
C     *** ncen = 0/1 for no/a center of inversion present in the cell
C     *** nsym = number of symetry operators (1<=nsym<=48)
C     *** nsub = number of sub-translations in the cell because of 
C     ***        a non-primitive Bravais lattice: I,A,B,C,F,R (1<=nsub<=4)
C     *** IROT() = array of all symmetry operators' rotation matrices
C     *** RTRS() = array of corresponding translation vectors
C     *** RSUB() = transl. vectors belonging to cell-sub-translations
C
      IMPLICIT none
      integer*4 nasym,masym,ia
      integer isw,mpix,nsym,ncen,nsub,mref,nref
      real*8 rtasym,rdint,pi,twopi,phs
C     *** density array ***
      real*8 RDEN(masym),RASYM(masym)
C     *** pixels along a,b,c axis; ia,ib,ic of pixels in asymm. unit ***
      integer NPIX(3),IASYM(3,masym)
C     *** symmetry operators ***
      integer IROT(3,3,48)
      real*8 RTRS(3,48),RSUB(3,4)
C     *** (hkl) of all given reflections ***
      integer IDX(3,mref)
C     *** cosine- and sine-tables ***
      real*4 RMIC(mref,masym)
CENT $      ,RMIS(mref,masym)
C     *** auxilary array ***
      integer IDE(0:mpix,0:mpix,0:mpix)
      integer kx,ky,kz,ic,it,is,nx,ny,nz,iref
      real*8 x,y,z,rx,ry,rz
      twopi=2*pi
C
C*** MAKE ASYMMETRIC UNIT COODINATE ***
C
      print *,'Calculating asymmetric unit pixels'
      IF (ISW.EQ.0) THEN
      nasym=0
      DO 10 KZ=0,NPIX(3)-1
      DO 10 KY=0,NPIX(2)-1
      DO 10 KX=0,NPIX(1)-1
      IF (IDE(KX,KY,KZ).GT.-1) THEN
	x=kx/FLOAT(npix(1))
	y=ky/FLOAT(npix(2))
	z=kz/FLOAT(npix(3))
        DO 20 IC=1,NCEN+1
        DO 20 IT=1,NSUB
        DO 20 IS=1,NSYM
        rx=IROT(1,1,IS)*x+IROT(1,2,IS)*y+IROT(1,3,IS)*z+RTRS(1,IS)
        ry=IROT(2,1,IS)*x+IROT(2,2,IS)*y+IROT(2,3,IS)*z+RTRS(2,IS)
        rz=IROT(3,1,IS)*x+IROT(3,2,IS)*y+IROT(3,3,IS)*z+RTRS(3,IS)
        IF (IC.EQ.2) THEN
          NX=NINT( DMOD(2-rx+RSUB(1,IT),1.0d0)*NPIX(1) )
          NY=NINT( DMOD(2-ry+RSUB(2,IT),1.0d0)*NPIX(2) )
          NZ=NINT( DMOD(2-rz+RSUB(3,IT),1.0d0)*NPIX(3) )
	ELSE
          NX=NINT( DMOD(2+rx+RSUB(1,IT),1.0d0)*NPIX(1) )
          NY=NINT( DMOD(2+ry+RSUB(2,IT),1.0d0)*NPIX(2) )
          NZ=NINT( DMOD(2+rz+RSUB(3,IT),1.0d0)*NPIX(3) )
        END IF
        IF (KX.NE.NX.OR.KY.NE.NY.OR.KZ.NE.NZ) IDE(NX,NY,NZ)=-1
   20   CONTINUE
        nasym=nasym+1
        IF (nasym.GT.MASYM) CALL PROEND(
     $   'DIMENSION OVER FOR MKASYM - TOO MANY POINTS IN ASYM. UNIT')
        RDEN(nasym)=rdint
        IASYM(1,nasym)=KX
        IASYM(2,nasym)=KY
        IASYM(3,nasym)=KZ
      END IF
   10 CONTINUE
      END IF
C
C*** MAKE PHASE FACTOR TABLES ***
C
      print *,'Calculating phase-factor tables'
      RTASYM=DFLOAT(NSUB*NSYM*(NCEN+1))
      DO 30 KX=0,NPIX(1)-1
      DO 30 KY=0,NPIX(2)-1
      DO 30 KZ=0,NPIX(3)-1
         IDE(KX,KY,KZ)=0
   30 CONTINUE
      DO 40 IC=1,NCEN+1
      DO 40 IT=1,NSUB
      DO 40 IS=1,NSYM
      DO 40 IA=1,nasym
	x=iasym(1,ia)/DFLOAT(npix(1))
	y=iasym(2,ia)/DFLOAT(npix(2))
	z=iasym(3,ia)/DFLOAT(npix(3))
        rx=IROT(1,1,IS)*x+IROT(1,2,IS)*y+IROT(1,3,IS)*z+RTRS(1,IS)
        ry=IROT(2,1,IS)*x+IROT(2,2,IS)*y+IROT(2,3,IS)*z+RTRS(2,IS)
        rz=IROT(3,1,IS)*x+IROT(3,2,IS)*y+IROT(3,3,IS)*z+RTRS(3,IS)
        IF (IC.EQ.2) THEN
          rx= DMOD(2-rx+RSUB(1,IT),1.0d0)
          ry= DMOD(2-ry+RSUB(2,IT),1.0d0)
          rz= DMOD(2-rz+RSUB(3,IT),1.0d0)
	ELSE
          rx= DMOD(2+rx+RSUB(1,IT),1.0d0)
          ry= DMOD(2+ry+RSUB(2,IT),1.0d0)
          rz= DMOD(2+rz+RSUB(3,IT),1.0d0)
        END IF
	NX=NINT( rx*npix(1) )
	NY=NINT( ry*npix(2) )
	NZ=NINT( rz*npix(3) )
        IDE(NX,NY,NZ)=IDE(NX,NY,NZ)+1
        DO 50 iref=1,NREF
          phs=twopi*(rx*IDX(1,iref)+ry*IDX(2,iref)+rz*IDX(3,iref))  
          RMIC(iref,IA)=RMIC(iref,IA)+ COS(phs)
CENT      RMIS(iref,IA)=RMIS(iref,IA)+ SIN(phs)
   50   CONTINUE
   40 CONTINUE
      DO 60 IA=1,nasym
         RASYM(IA)=DFLOAT(IDE(IASYM(1,IA),IASYM(2,IA),IASYM(3,IA)))
   60 CONTINUE
      print *,'... finished'
C
      RETURN
      END
